"use strict";
var nodeIdToTextNode = {};
var ClickBubble = (function () {
    function ClickBubble() {
        var self = this;
        self.downtime = Date.now();
        self.element = document.createElement('div');
        self.element.classList.add('click-bubble');
        setTimeout(function () {
            self.element.classList.add('down');
        });
    }
    ClickBubble.prototype.up = function () {
        var self = this;
        setTimeout(function () {
            self.element.classList.remove('down');
            if (self.element.parentNode) {
                self.element.parentNode.removeChild(self.element);
            }
        }, Math.max(100 - (Date.now() - this.downtime), 0));
    };
    return ClickBubble;
}());
function isElement(node) {
    return node && node.setAttribute !== undefined;
}
function isTextInput(node) {
    return node && node.value !== undefined;
}
var SiqStoryTellerCtrl = (function () {
    function SiqStoryTellerCtrl($element, LibrarySrvc) {
        this.storyIndex = 0;
        var self = this;
        self.iframe = $element.find('iframe')[0];
        self.element = $element.find('.story-teller')[0];
        self.pointer = $element.find('.story-teller-mouse-cursor')[0];
        // clear it out cause it's easier to go from the root
        var idocument = self.iframe.contentDocument;
        idocument.open();
        idocument.write("<!DOCTYPE html>");
        idocument.write("<html>");
        idocument.write("<head></head>");
        idocument.write("<body>this is the iframe</body>");
        idocument.write("</html>");
        idocument.close();
        self.iframe.contentDocument.removeChild(self.iframe.contentDocument.childNodes[1]);
        LibrarySrvc.getLatestStory().then(function (story) {
            self.story = story;
            self.playNextStoryFrame();
        });
    }
    SiqStoryTellerCtrl.prototype.playNextStoryFrame = function () {
        // put this in a timeout just to get freaking stack traces ANGULAR
        var self = this;
        var thisTwist = self.story[self.storyIndex];
        if (!thisTwist) {
            return;
        }
        var lastTwist = self.story[self.storyIndex - 1];
        var nextFrameDelay = Math.min(Math.ceil(thisTwist.timeSincePageLoad - (lastTwist && lastTwist.timeSincePageLoad || 0)), 1000);
        setTimeout(function () {
            var twist = thisTwist;
            var targetNode = twist.targetNode && self.findNodeByNodeId(twist.targetNode.nodeId);
            switch (twist.type) {
                case 'childList':
                    if (twist.addedNodes) {
                        twist.addedNodes.forEach(function (storyNode) {
                            if (!targetNode) {
                                console.warn('could not find targetNode for addition', JSON.stringify(twist.targetNode));
                                return;
                            }
                            var node = self.createNode(storyNode);
                            if (node) {
                                targetNode.appendChild(node);
                            }
                            else if (storyNode.nodeType === 1 || storyNode.nodeType === 3) {
                                throw new Error('couldnt make node for element or text node');
                            }
                        });
                    }
                    if (twist.removedNodes) {
                        twist.removedNodes.forEach(function (storyNode) {
                            if (!targetNode) {
                                console.log('could not find targetNode for removal', JSON.stringify(twist.targetNode));
                                return;
                            }
                            var removeNode;
                            if (storyNode.nodeType === 3 || storyNode.nodeType === 8) {
                                removeNode = self.findTextNode(storyNode, targetNode);
                                delete nodeIdToTextNode[storyNode.nodeId];
                            }
                            else {
                                removeNode = self.findNodeByNodeId(storyNode.nodeId, targetNode);
                            }
                            if (removeNode) {
                                if (removeNode.parentNode !== targetNode) {
                                    console.log('removeNode isnt the child of the target at this point....', storyNode);
                                    return;
                                }
                                targetNode.removeChild(removeNode);
                            }
                        });
                    }
                    break;
                case 'attributes':
                    if (isElement(targetNode)) {
                        targetNode.setAttribute(twist.attributeName, twist.attributeValue);
                    }
                    break;
                case 'resize':
                    self.element.style.width = (self.iframe.width = twist.width.toString()) + 'px';
                    self.element.style.height = (self.iframe.height = twist.height.toString()) + 'px';
                    break;
                case 'event':
                    switch (twist.eventType) {
                        case 'mousemove':
                            var top = twist.clientY + 'px';
                            var left = twist.clientX + 'px';
                            self.pointer.style.top = top;
                            self.pointer.style.left = left;
                            if (self.currentClickBubble) {
                                self.currentClickBubble.element.style.top = top;
                                self.currentClickBubble.element.style.left = left;
                            }
                            break;
                        case 'mousedown':
                            self.pointer.classList.add('mousedown');
                            var top = twist.clientY + 'px';
                            var left = twist.clientX + 'px';
                            self.currentClickBubble = new ClickBubble();
                            self.currentClickBubble.element.style.top = top;
                            self.currentClickBubble.element.style.left = left;
                            self.element.appendChild(self.currentClickBubble.element);
                            break;
                        case 'mouseup':
                            self.pointer.classList.remove('mousedown');
                            self.currentClickBubble.up();
                            self.currentClickBubble = null;
                            break;
                        case 'input':
                            if (isTextInput(targetNode)) {
                                targetNode.value = twist.textValue;
                            }
                            break;
                    }
                    break;
            }
            self.storyIndex++;
            self.playNextStoryFrame();
        }, nextFrameDelay);
    };
    SiqStoryTellerCtrl.prototype.createNode = function (storyNode) {
        if (storyNode.nodeType === 3 || storyNode.nodeType === 8) {
            if (nodeIdToTextNode[storyNode.nodeId]) {
                return nodeIdToTextNode[storyNode.nodeId];
            }
            var textNode;
            if (storyNode.nodeType === 3) {
                textNode = this.iframe.contentDocument.createTextNode(storyNode.nodeValue);
            }
            else if (storyNode.nodeType === 8) {
                textNode = this.iframe.contentDocument.createComment(storyNode.nodeValue);
            }
            textNode['__siqStoryNodeId'] = storyNode.nodeId;
            nodeIdToTextNode[storyNode.nodeId] = textNode;
            return textNode;
        }
        if (storyNode.nodeType === 1) {
            var node_1 = this.findNodeByNodeId(storyNode.nodeId);
            if (!node_1) {
                node_1 = this.iframe.contentDocument.createElement(storyNode.tagName);
                node_1.setAttribute('siq-story-node-id', storyNode.nodeId);
                if (storyNode.attributes) {
                    Object.keys(storyNode.attributes).forEach(function (attributeName) {
                        if (attributeName === 'siqStoryCSS') {
                            node_1.innerHTML = storyNode.attributes[attributeName];
                        }
                        else {
                            try {
                                node_1.setAttribute(attributeName, storyNode.attributes[attributeName]);
                            }
                            catch (e) {
                                console.log('got weird attribute', attributeName, storyNode.attributes[attributeName]);
                            }
                        }
                    });
                }
            }
            return node_1;
        }
    };
    SiqStoryTellerCtrl.prototype.findTextNode = function (storyNode, targetNode) {
        if (nodeIdToTextNode[storyNode.nodeId]) {
            return nodeIdToTextNode[storyNode.nodeId];
        }
    };
    SiqStoryTellerCtrl.prototype.findNodeByNodeId = function (nodeId, ancestorNode) {
        if (nodeId === 'document') {
            return this.iframe.contentDocument;
        }
        if (nodeId === 'body') {
            return this.iframe.contentDocument.body;
        }
        if (nodeId !== undefined) {
            ancestorNode = ancestorNode || this.iframe.contentDocument;
            return ancestorNode.querySelector('[siq-story-node-id="' + nodeId + '"]');
        }
    };
    return SiqStoryTellerCtrl;
}());
exports.SiqStoryTellerCtrl = SiqStoryTellerCtrl;
//# sourceMappingURL=SiqStoryTellerCtrl.js.map